from brasilapy.src.interfaces import IAdapter, IApiRoute


class RoutesAdapter(IAdapter):
    """create routes from a source."""

    @staticmethod
    def handle(base_url: str, api_paths: dict[str, str], route: IApiRoute) -> list[IApiRoute]:
        """ generates the client routes.

        Params:
            base_url <string>: API standard path.
            api_paths <dictionary<string, string>>: 
                API route structures (the keys are the endpoint name and the values are the endpoint path). 
        
        Returns <list<IApiRoute>>: 
            list of "IApiRoute" class objects.
        """

        return [route(name, base_url + path) for name, path in api_paths.items()]



from abc import ABC, abstractmethod
from typing import Any


class IApiRoute(ABC):
    """API route interface."""

    @abstractmethod
    def request(*args: Any) -> Any:
        """API requester."""
        


from abc import ABC, abstractstaticmethod
from typing import Any


class IAdapter(ABC):
    """adapter interface"""

    @abstractstaticmethod
    def handler(*args: Any) -> Any:
        """data adapter handler."""



from brasilapy.src.interfaces import IAdapter


class StringFormatter(IAdapter):
    """remove chars from string."""

    @staticmethod
    def handler(text: str, banned_chars: list[str] | str) -> str:
        """ Handles string data.

        Params:
            text <string>: text to modify.
            banned_chars <list<string> | string>: characters that will be removed.
        
        Returns <string>:
            formatted string without unwanted chars.
        """

        return ''.join(char for char in text if char not in banned_chars)



from .entities import ApiRouteGet
from .adapters import RoutesAdapter
from .errors import RouteNotFoundError
from typing import Callable

API_BASE_URL = 'https://brasilapi.com.br/api'
API_GET_ROUTES = {
    'get_banks': '/banks/v1/',
    'get_bank': '/banks/v1/{code}',
    'get_cep_v1': '/cep/v1/{cep}',
    'get_cep_v2': '/cep/v2/{cep}',
    'get_cnpj': '/cnpj/v1/{cnpj}',
    'get_ddd': '/ddd/v1/{ddd}',
    'get_feriados_nacionais': '/feriados/v1/{ano}',
    'get_fipe_veiculo': '/fipe/marcas/v1/{tipoVeiculo}',
    'get_fipe_precos': '/fipe/preco/v1/{codigoFipe}',
    'get_fipe_tabelas': '/banks/v1/',
    'get_ibge_municipios': '/ibge/municipios/v1/{siglaUF}?providers=dados-abertos-br,gov,wikipedia',
    'get_ibge_estados': '/ibge/uf/v1',
    'get_ibge_estado': '/ibge/uf/v1/{code}',
    'get_registro_br': '/regsitrobr/v1/{domain}',
    'get_taxas': '/taxas/v1',
    'get_taxa': '/taxas/v1/{sigla}'
}


class BrasilApiClient(object):
    """Brasil API client."""

    def __init__(self) -> None:
        self.routes = RoutesAdapter.handle(API_BASE_URL, API_GET_ROUTES, ApiRouteGet)
    
    def __call__(self) -> None:
        for route in self.route:
            setattr(self, route.name, route.request)
    
    def __getattr__(self, name: str) -> Callable:
        """ route endpoint requester.
        Params:
            name <string>: endpoint name.

        Exceptions:
            RouteNotFoundError: throws when the called method do not exists.

        Returns <Callable>:
            method of the <IApiRoute>.
        """
        for route in self.routes:
            if route.name == name:
                return route.request
        
        raise RouteNotFoundError(f'route "{name}" do not exists.')



import requests
import re
from brasilapy.src.dtos import ApiResponseDTO
from brasilapy.src.interfaces import IApiRoute
from brasilapy.src.errors  import ParameterError
from brasilapy.src.adapters import StringFormatter


class ApiRouteGet(IApiRoute):
    """API route GET requester method."""

    def __init__(self, name: str, url: str) -> None:
        """
        Params:
            name <string>: route name. 
            url: <string>: request url endpoint.
        """
        
        self.name = name
        self.url = url
 
    def request(self, parameter_value: str = None, query: dict[str, str] = {}) -> ApiResponseDTO:
        """makes a get request.

        Params:
            parameter_value <string>: request path parameter value.
            query: <dictionary<string, string>>: http query parameters.
  
        Exceptions:
            TypeError: throws when "parameter_name" is not a instance of string type.
            ParameterError: throws when the parameter is invalid (when it exists or not).

        Returns <ApiResponseDTO>:
            response model.
        """

        request_url = self.url
        parameter_name = None
            
        if parameters_list := re.findall("[{][a-zA-Z]{1,}[}]", self.url):
            parameter_name = StringFormatter.handler(parameters_list.pop(), ['{', '}'])

        if parameters_list and parameter_value:
            raise ParameterError('request there are no parameters.')

        if parameter_name:
            if not isinstance(parameter_value, str):
                raise TypeError(f'parameter "{parameter_name}" must be a string.')

            if not parameter_value:
                raise ParameterError(f'parameter "{parameter_name}" is required.')
            
            request_url = self.url.format_map({ parameter_name: parameter_value })

        requester = requests.get(request_url, params=query)
        
        return ApiResponseDTO(
            headers=requester.headers,
            status=requester.status_code, 
            content=requester.json()
        )
        


import setuptools

with open("./brasilapy/README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="brasilapy", 
    packages=setuptools.find_packages(),                    # This is the name of the package
    version="0.0.5",                        # The initial release version
    author="Filipe Soares",                     # Full name of the author
    description="Brasil API client.",
    license='MIT',
    long_description=long_description,      # Long description read from the the readme file
    long_description_content_type="text/markdown",
    url='https://github.com/lipe14-ops/brasilapy',
    author_email='filipe.ns1001@gmail.com',   # List of all python modules to be installed
    keywords=['api', 'requests', 'brasilapy', 'web', 'client' ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],                                      # Information to filter the project on PyPi website
    python_requires='>=3.9',                # Minimum version requirement of the package   # Directory of the source code of the package
    install_requires=[]                     # Install other dependencies if any
)



from dataclasses import dataclass


@dataclass(frozen=True)
class ApiResponseDTO:
    """API response model.

    Params:
        header <dictionary<string, any>>: http header.
        status <integer>:  response status code.
        content <dictionary<string, any>>: API response. 
    """
    
    headers: dict
    status: int
    content: dict
